@@ -901,6 +901,7 @@ class TestAPI(BaseTest):
                      "filename!*/files/",
                      "filename^*data/",
                      "filename$*.txt",
+                     "filename=~RE[A-Z]*\\.tx",
                      "length>250623",
                      "length>=250624",
                      "length<250625",
@@ -911,6 +912,7 @@ class TestAPI(BaseTest):
                      "filename!*/examples/",
                      "filename^*dat/",
                      "filename$*.tx",
+                     "filename=~^READ.*\\.txt",
                      "length>250624",
                      "length>=250625",
                      "length<250624",
